package jalview.jbgui;

/*
 * bsoares 2022-04-18
 * ClassicJInternalFrame is a hacked fix for ClassicJInternalFrame's menu (and JComboBox)
 * widgets not retaining focus when clicked on prior to the frame having focus (e.g.
 * if another frame has focus and you click on the menu of an unfocused frame then
 * the menu doesn't open (or more accurately does open and immediately closes again).
 * This is well documented Java 11+ behaviour, see
 * https://bugs.openjdk.java.net/browse/JDK-8274931
 * A workaround is to reinstate the old setSelected() and dispose() methods that do
 * not send out an ungrab event.  This is taken from
 * https://github.com/george-hawkins/jinternalframe-bug/blob/main/src/main/java/com/example/InternalFrameBugFixDemo.java
 * 
 * We now extend GAlignFrame from jalview.jbgui.ClassicJInternalFrame.
 */

import java.beans.BeanProperty;
import java.beans.PropertyVetoException;

import javax.swing.JInternalFrame;
import javax.swing.event.InternalFrameEvent;

public class ClassicJInternalFrame extends JInternalFrame
{
  public ClassicJInternalFrame()
  {
    super();
  }

  public ClassicJInternalFrame(String title)
  {
    super(title);
  }

  public ClassicJInternalFrame(String title, boolean resizable)
  {
    super(title, resizable);
  }

  public ClassicJInternalFrame(String title, boolean resizable,
          boolean closable)
  {
    super(title, resizable, closable);
  }

  public ClassicJInternalFrame(String title, boolean resizable,
          boolean closable, boolean maximizable)
  {
    super(title, resizable, closable, maximizable);
  }

  public ClassicJInternalFrame(String title, boolean resizable,
          boolean closable, boolean maximizable, boolean iconifiable)
  {
    super(title, resizable, closable, maximizable, iconifiable);
  }

  // -----------------------------------------------------------------

  // The two methods below, `setSelected` and `dispose`, are copied from
  // `ClassicJInternalFrame` as it existed here:
  // https://github.com/openjdk/jdk/blob/jdk-11%2B12/src/java.desktop/share/classes/jalview.jbgui.ClassicJInternalFrame.java
  // They've then been modified to back out this change:
  // https://github.com/openjdk/jdk/commit/4b8cfe5a6058cdada1ad6efbb8f81e2f8f53a177
  // The ticket for that change is
  // https://bugs.openjdk.java.net/browse/JDK-8173739
  //
  // On moving to Java 11 we found that if you had two windows, A and B, where A
  // currently has focus and
  // B has a combo-box, if you click on the combo-box of B you see the combo's
  // dropdown list appear
  // and then immediately disappear - the trigger for this disappearance is an
  // ungrab events generated
  // by A, i.e. the window losing focus as a result of you clicking on a
  // component in window B.
  // The generation of ungrab events in this situation was introduced in the
  // change linked to above.

  @BeanProperty(
    description = "Indicates whether this internal frame is currently the active frame.")
  public void setSelected(boolean selected) throws PropertyVetoException
  {
    // The InternalFrame may already be selected, but the focus
    // may be outside it, so restore the focus to the subcomponent
    // which previously had it. See Bug 4302764.
    if (selected && isSelected)
    {
      restoreSubcomponentFocus();
      return;
    }
    // The internal frame or the desktop icon must be showing to allow
    // selection. We may deselect even if neither is showing.
    if ((isSelected == selected) || (selected
            && (isIcon ? !desktopIcon.isShowing() : !isShowing())))
    {
      return;
    }

    Boolean oldValue = isSelected ? Boolean.TRUE : Boolean.FALSE;
    Boolean newValue = selected ? Boolean.TRUE : Boolean.FALSE;
    fireVetoableChange(IS_SELECTED_PROPERTY, oldValue, newValue);

    /* We don't want to leave focus in the previously selected
       frame, so we have to set it to *something* in case it
       doesn't get set in some other way (as if a user clicked on
       a component that doesn't request focus).  If this call is
       happening because the user clicked on a component that will
       want focus, then it will get transfered there later.
       We test for parent.isShowing() above, because AWT throws a
       NPE if you try to request focus on a lightweight before its
       parent has been made visible */

    if (selected)
    {
      restoreSubcomponentFocus();
    }

    isSelected = selected;
    firePropertyChange(IS_SELECTED_PROPERTY, oldValue, newValue);
    if (isSelected)
      fireInternalFrameEvent(InternalFrameEvent.INTERNAL_FRAME_ACTIVATED);
    else
    {
      fireInternalFrameEvent(InternalFrameEvent.INTERNAL_FRAME_DEACTIVATED);
    }
    repaint();
  }

  public void dispose()
  {
    if (isVisible())
    {
      setVisible(false);
    }
    if (!isClosed)
    {
      firePropertyChange(IS_CLOSED_PROPERTY, Boolean.FALSE, Boolean.TRUE);
      isClosed = true;
    }
    fireInternalFrameEvent(InternalFrameEvent.INTERNAL_FRAME_CLOSED);
  }
}
